from __future__ import annotations

from .AFXGuiCommand import AFXGuiCommand
from .AFXGuiObjectManager import AFXGuiObjectManager
from .AFXMode import AFXMode
from .FXObject import FXObject


class AFXGuiMode(AFXMode):
    """This class is the abstract base class for modes."""

    #: Used to destroy dialogs.
    ID_DESTROY_DIALOGS: int = hash("ID_DESTROY_DIALOGS")

    #: Used to handle bailout.
    ID_HANDLE_BAILOUT: int = hash("ID_HANDLE_BAILOUT")

    def __init__(self, owner: AFXGuiObjectManager):
        """Constructor.

        Parameters
        ----------
        owner : AFXGuiObjectManager
            Owner (a module or a toolset) of the mode.
        """

    def activate(self):
        """Performs the necessary tasks when the mode is activated.

        Reimplemented in AFXForm, and AFXProcedure.
        """

    def cancel(self, tgt: FXObject | None = None, msg: int = 0):
        """Requests a cancellation of the mode. When the cancel operation completes, successfully or not, the
        target will be sent the given message. The message data pointer will be non-zero for successful
        cancellation and zero if the cancel operation was abandoned for some purpose.

        Parameters
        ----------
        tgt : FXObject | None
            Completion message target.
        msg : int
            Completion message ID.
        """

    def commit(self):
        """Performs the necessary tasks when a dialog box of the mode commits.

        Implemented in AFXForm, and AFXProcedure.
        """

    def continueMode(self):
        """Virtual method that must be defined in the derived class--used to get the next dialog box or step in
        the mode.

        Implemented in AFXForm, and AFXProcedure.
        """

    def deactivate(self):
        """Performs the necessary tasks when the mode is deactivated.

        Reimplemented in AFXProcedureForm, and AFXProcedure.
        """

    def destroyDialogs(self):
        """Destroys the associated dialogs when the mode is deactivated.

        The exact behavior of this function is controlled by the return value of dialog box's
        getDeactivateAction() method.
        """

    def doCustomChecks(self):
        """This class provides an empty implementation of this method; the derived class should redefine this
        method to perform its specific verifications on the user input."""

    def doCustomTasks(self):
        """This class provides an empty implementation of this method, which is called after the commands are
        processed successfully.

        The derived class should redefine this method to perform its specific tasks such as writing a
        confirmation message to the main window.
        """

    def getCommandString(self):
        """Returns a string containing a list of commands generated by each command associated with the mode."""

    def getCurrentDialog(self):
        """Returns the dialog box currently posted by the mode (may be NULL)."""

    def getModeName(self):
        """Returns the mode's name."""

    def getOwner(self):
        """Returns the mode's owner (a module or a tool set)."""

    def getPressedButtonId(self):
        """Returns the ID of the button that the user pressed in the currently posted dialog box."""

    def handleException(self, exc):
        """Posts a dialog box with an error message for the given exception. The derived class should redefine
        this method if it needs to handle the exception differently.

        Parameters
        ----------
        exc : nex_Exception
            Exception.
        """

    def handleKeywordException(self, exc):
        """Posts a dialog box with an error message for the given exception thrown by a keyword during data
        validation.

        Parameters
        ----------
        exc : nex_Exception
            Exception.
        """

    def isKeyword(self, object: FXObject):
        """Returns True if the object is one of the mode's keywords.

        Parameters
        ----------
        object : FXObject
        """

    def issueCommands(self, writeToReplay: bool = True, writeToJournal: bool = False):
        """Generates commands based on the current state, sends the commands, and deactivates the mode if
        necessary. If the commands did not complete successfully, a dialog box will be posted with an error
        message.

        Parameters
        ----------
        writeToReplay : bool
            True if commands should be written to the replay file; False if not.
        writeToJournal : bool
            True if commands should be written to the journal file; False if not.
        """

    def okToCancel(self):
        """Method indicating if the dialog box should be cancelled if the context changes (for example, if the
        user opens a new database); the base class implementation returns True indicating it is OK to cancel the
        mode, derived classes can overwrite this method and return False to prevent the mode from being canceled
        during a context change."""

    def onKeywordsUpdated(self, command: AFXGuiCommand):
        """Hook method called whenever the command's object changes in Kernel. By default this function does
        nothing.

        Parameters
        ----------
        command : AFXGuiCommand
            Command for which the query fired.
        """

    def registerDefaultsObject(self, command: AFXGuiCommand, objectName: str):
        """Registers the defaults object that will be queried when the Defaults button in the dialog box is
        pressed.

        Parameters
        ----------
        command : AFXGuiCommand
            Command whose keywords to be populated with the defaults object values.
        objectName : str
            Name of defaults object to be queried.
        """

    def registerQueries(self):
        """Registers queries with the GUI infrastructure for the mode's active commands and unregisters queries
        for the inactive ones."""

    def removeAllProxies(self):
        """Unregisters queries and deletes all the associated proxies."""

    def sendCommandString(self, command: str, writeToReplay: bool = True, writeToJournal: bool = False):
        """Sends the given command string (which can contain multiple commands, separated by command
        delimiters).

        Parameters
        ----------
        command : str
            Command string.
        writeToReplay : bool
            True if commands should be written to the replay file; False if not.
        writeToJournal : bool
            True if commands should be written to the journal file; False if not.
        """

    def setModeName(self, name: str):
        """Sets the mode's name.

        Parameters
        ----------
        name : str
        """

    def unregisterAllQueries(self):
        """Unregisters queries with the GUI infrastructure for all the mode's commands."""

    def verify(self):
        """Verifies each active command associated with the mode and throws an exception if any command contains
        keywords with invalid data."""

    def verifyCurrentKeywordValues(self):
        """Checks whether keywords of active commands for the current dialog box contain valid data and, if not,
        posts a dialog box with an error message.

        Reimplemented in AFXProcedure.
        """

    def verifyKeywordValues(self):
        """Checks whether keywords of active commands contain valid data and, if not, posts a dialog box with an
        error message."""



from __future__ import annotations

from typing import List, Sequence, Union

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from .MeshEdge import MeshEdge


@abaqus_class_doc
class MeshEdgeArray(List[MeshEdge]):
    """The MeshEdgeArray is a sequence of MeshEdge objects.

    .. note::
        This object can be accessed by::

            import part
            mdb.models[name].parts[name].elementEdges
            import assembly
            mdb.models[name].rootAssembly.allInstances[name].elementEdges
            mdb.models[name].rootAssembly.instances[name].elementEdges
    """

    @abaqus_method_doc
    def __init__(self, elemEdges: list[MeshEdge]):
        """This method creates a MeshEdgeArray object.

        .. note::
            This function can be accessed by::

                mesh.MeshEdgeArray

        Parameters
        ----------
        elemEdges
            A list of MeshEdge objects.

            .. versionchanged:: 2022
                The argument ``edges`` was renamed to ``elemEdges``.

        Returns
        -------
        MeshEdgeArray
            A MeshEdgeArray object.
        """
        super().__init__()

    @abaqus_method_doc
    def getSequenceFromMask(self, mask: Union[str, Sequence[str]]) -> MeshEdgeArray:
        """This method returns the objects in the MeshEdgeArray identified using the specified
        **mask**. When large number of objects are involved, this method is highly efficient.

        Parameters
        ----------
        mask
            A String specifying the object or objects.

        Returns
        -------
        MeshEdgeArray
            A MeshEdgeArray object.

        Raises
        ------
        Error
            The mask results in an empty sequence, An exception occurs if the resulting sequence is empty.
        """
        return MeshEdgeArray([MeshEdge()])

    @abaqus_method_doc
    def getMask(self):
        """This method returns a string specifying the object or objects.

        Returns
        -------
        str
            A String specifying the object or objects.
        """
        return ""



from abqpy.decorators import abaqus_class_doc

from ..UtilityAndView.abaqusConstants import OFF, Boolean


@abaqus_class_doc
class OdbDiagnosticAttempt:
    """The OdbDiagnosticAttempt object.

    .. note::
        This object can be accessed by::

            import visualization
            session.odbData[name].diagnosticData.steps[i].increments[i].attempts[i]
    """

    #: A boolean specifying the state of Auto-stablilization. This attribute is read-only.
    autoStabilize: Boolean = OFF

    #: A boolean specifying the state of convergence for the attempt. This attribute is
    #: read-only.
    isConverged: Boolean = OFF

    #: A boolean specifying the state of cutback. This attribute is read-only.
    isCutBack: Boolean = OFF

    #: A boolean specifying whether or not reordering is needed. This attribute is read-only.
    needsReordering: Boolean = OFF

    #: An int specifying the number of cutback diagnostics. This attribute is read-only.
    numberOfCutbackDiagnostics: str = ""

    #: An int specifying the number of iterations for the particular attempt. This attribute is
    #: read-only.
    numberOfIterations: str = ""

    #: An int specifying the number of iterations with severe discontinuities This attribute is
    #: read-only.
    numberOfSevereDiscontinuityIterations: str = ""

    #: A float specifying the size of the increment of the particular attempt. This attribute
    #: is read-only.
    size: str = ""



from __future__ import annotations

from abqpy.decorators import abaqus_class_doc

from ..Region.Region import Region


@abaqus_class_doc
class IMAField:
    """A IMAField is an object used to define material instance name volume fractions for the MaterialAssignment
    predefined field.

    .. note::
        This object can be accessed by::

            import load
            mdb.models[name].predefinedFields[name].fieldList
    """

    #: A Region object specifying the sub-region of the selected part instance to which the
    #: volume fractions will be applied.
    region: Region = Region()

    #: A tuple of Strings specifying the name of the discrete fields that contain the volume
    #: fraction data. The length of the tuple corresponds to the number of material instance
    #: names, as established by the assigned Eulerian section.
    discfieldList: tuple[str, ...] = ()



from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ....UtilityAndView.abaqusConstants import OFF, RETENTION_FACTOR, Boolean
from ....UtilityAndView.abaqusConstants import abaqusConstants as C


@abaqus_class_doc
class BrittleShear:
    """The BrittleShear object specifies the postcracking shear behavior of a material used in a brittle
    cracking model.

    .. note::
        This object can be accessed by::

            import material
            mdb.models[name].materials[name].brittleCracking.brittleShear
            import odbMaterial
            session.odbs[name].materials[name].brittleCracking.brittleShear

        The table data for this object are:

        - If **type** = RETENTION_FACTOR the table data specify the following:

            - Shear retention factor.
            - Crack opening strain.
            - Temperature, if the data depend on temperature.
            - Value of the first field variable, if the data depend on field variables.
            - Value of the second field variable.
            - Etc.
        - If **type** = POWER_LAW the table data specify the following:

            - :math:`e`.
            - :math:`p`.
            - Temperature, if the data depend on temperature.
            - Value of the first field variable, if the data depend on field variables.
            - Value of the second field variable.
            - Etc.

        The corresponding analysis keywords are:

        - BRITTLE SHEAR
    """

    @abaqus_method_doc
    def __init__(
        self,
        table: tuple,
        temperatureDependency: Boolean = OFF,
        dependencies: int = 0,
        type: Literal[C.POWER_LAW, C.RETENTION_FACTOR] = RETENTION_FACTOR,
    ):
        """This method creates a BrittleShear object.

        .. note::
            This function can be accessed by::

                mdb.models[name].materials[name].brittleCracking.BrittleShear
                session.odbs[name].materials[name].brittleCracking.BrittleShear

        Parameters
        ----------
        table
            A sequence of sequences of Floats specifying the items described below.
        temperatureDependency
            A Boolean specifying whether the data depend on temperature. The default value is OFF.
        dependencies
            An Int specifying the number of field variable dependencies. The default value is 0.
        type
            A SymbolicConstant specifying the type of postcracking shear behavior. Possible values
            are RETENTION_FACTOR and POWER_LAW. The default value is RETENTION_FACTOR.

        Returns
        -------
        BrittleShear
            A BrittleShear object.

        Raises
        ------
        RangeError
        """
        ...

    @abaqus_method_doc
    def setValues(self, *args, **kwargs):
        """This method modifies the BrittleShear object.

        Raises
        ------
        RangeError
        """
        ...



from __future__ import annotations

from abqpy.decorators import abaqus_class_doc


@abaqus_class_doc
class RayleighDampingByFrequencyComponent:
    """A RayleighDampingByFrequencyComponent object is used to define Rayleigh damping over a range of
    frequencies.

    .. note::
        This object can be accessed by::

            import step
            mdb.models[name].steps[name].rayleighDampingByFrequency.components[i]
    """

    #: A Float specifying the frequency value in cycles/time.
    frequency: float | None = None

    #: A Float specifying the mass proportional damping, αM.
    alpha: float | None = None

    #: A Float specifying the stiffness proportional damping, βM.
    beta: float | None = None



from __future__ import annotations

from .FXIcon import FXIcon
from .FXIconItem import FXIconItem

#: Show hidden files or directories.
FILELIST_SHOWHIDDEN: int = hash("FILELIST_SHOWHIDDEN")

#: Show only directories.
FILELIST_SHOWDIRS: int = hash("FILELIST_SHOWDIRS")

#: Do not create associations for files.
FILELIST_NO_OWN_ASSOC: int = hash("FILELIST_NO_OWN_ASSOC")


class FXFileItem(FXIconItem):
    """File item."""

    def __init__(self, text: str, bi: FXIcon | None = None, mi: FXIcon | None = None, ptr: str = "None"):
        """Constructor.

        Parameters
        ----------
        text : str

        bi : FXIcon | None

        mi : FXIcon | None

        ptr : str
        """

    def getDate(self):
        """Return the date for this item."""

    def getSize(self):
        """Return the file size for this item."""

    def isBlockdev(self):
        """Return True if this is a block device item."""

    def isChardev(self):
        """Return True if this is a character device item."""

    def isDirectory(self):
        """Return True if this is a directory item.

        Reimplemented in AFXFileItem.
        """

    def isExecutable(self):
        """Return True if this is an executable item."""

    def isFifo(self):
        """Return True if this is an FIFO item."""

    def isFile(self):
        """Return True if this is a file item."""

    def isSocket(self):
        """Return True if this is a socket."""

    def isSymlink(self):
        """Return True if this is a symbolic link item."""



from __future__ import annotations

from typing import Sequence

from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..DisplayGroup.DisplayGroup import DisplayGroup
from ..DisplayGroup.DisplayGroupInstance import DisplayGroupInstance
from ..DisplayGroup.Leaf import Leaf
from ..UtilityAndView.abaqusConstants import (
    EMPTY_LEAF,
    OFF,
    ON,
    WIREFRAME,
    Boolean,
    SymbolicConstant,
)
from ..UtilityAndView.abaqusConstants import abaqusConstants as C
from .BCDisplayOptions import BCDisplayOptions
from .ConstraintDisplayOptions import ConstraintDisplayOptions
from .EngineeringFeatureDisplayOptions import EngineeringFeatureDisplayOptions
from .GeometricRestrictionDisplayOptions import GeometricRestrictionDisplayOptions
from .GeometryDisplayOptions import GeometryDisplayOptions
from .InteractionDisplayOptions import InteractionDisplayOptions
from .LoadDisplayOptions import LoadDisplayOptions
from .MeshDisplayOptions import MeshDisplayOptions
from .OptimizationTaskDisplayOptions import OptimizationTaskDisplayOptions
from .PredefinedFieldDisplayOptions import PredefinedFieldDisplayOptions
from .StopConditionDisplayOptions import StopConditionDisplayOptions
from .SymbolDisplayOptions import SymbolDisplayOptions


@abaqus_class_doc
class AssemblyDisplayOptions:
    """The AssemblyDisplayOptions object stores settings that specify how assemblies are to be displayed in a
    particular viewport. The AssemblyDisplayOptions object has no constructor. When you create a new viewport,
    the settings are copied from the current viewport.

    .. note::
        This object can be accessed by::

            import assembly
            session.viewports[name].assemblyDisplay
            session.viewports[name].layers[name].assemblyDisplay
    """

    #: A Boolean specifying whether boundary conditions are shown. The default value is OFF.
    bcs: Boolean = OFF

    #: A Boolean specifying whether connectors are shown. The default value is OFF.
    connectors: Boolean = OFF

    #: A Boolean specifying whether the connector end points are shown. This member is
    #: applicable only if **connectors** = ON. The default value is ON.
    cnxEndPoints: Boolean = ON

    #: A Boolean specifying whether the connector local coordinate system axes are shown. This
    #: member is applicable only if **connectors** = ON. The default value is ON.
    cnxLocalAxes: Boolean = ON

    #: A Boolean specifying whether the connector section type labels are shown. This member is
    #: applicable only if **connectors** = ON. The default value is ON.
    cnxTypeLabels: Boolean = ON

    #: A Boolean specifying whether the tag information is displayed along with the connector
    #: section type labels. This member is applicable only if **connectors** = ON and if
    #: **cnxTypeLabels** = ON. The default value is OFF.
    cnxTagDisplay: Boolean = OFF

    #: A Boolean specifying whether constraints are shown. The default value is OFF.
    constraints: Boolean = OFF

    #: A Boolean specifying whether to display engineering features. The default value is OFF.
    engineeringFeatures: Boolean = OFF

    #: A Boolean specifying whether geometric restrictions are shown. The default value is OFF.
    geometricRestrictions: Boolean = OFF

    #: A Boolean specifying whether to render the beam profiles. The default value is OFF.
    renderBeamProfiles: Boolean = OFF

    #: A Float specifying the beam profile scale factor. The beamScaleFactor must be greater
    #: than zero. The default value is 1.0.
    beamScaleFactor: float = 1

    #: A Boolean specifying whether fields and initial conditions are shown. The default value
    #: is OFF.
    predefinedFields: Boolean = OFF

    #: A Boolean specifying whether interactions are shown. The default value is OFF.
    interactions: Boolean = OFF

    #: A Boolean specifying whether loads are shown. The default value is OFF.
    loads: Boolean = OFF

    #: A Boolean specifying whether the mesh is shown. The default value is OFF.
    mesh: Boolean = OFF

    #: A Boolean specifying whether optimization tasks are shown. The default value is OFF.
    optimizationTasks: Boolean = OFF

    #: A Boolean specifying whether stop conditions are shown. The default value is OFF.
    stopConditions: Boolean = OFF

    #: A SymbolicConstant specifying how the image in the viewport is rendered. Possible values
    #: are WIREFRAME, HIDDEN, SHADED, and FILLED. The default value is WIREFRAME.
    renderStyle: SymbolicConstant = WIREFRAME

    #: A BCDisplayOptions object.
    bcOptions: BCDisplayOptions = BCDisplayOptions()

    #: A ConstraintDisplayOptions object.
    constraintOptions: ConstraintDisplayOptions = ConstraintDisplayOptions()

    #: A DisplayGroup object specifying the current display group and referring to an object in
    #: the **displayGroups** member of Session.
    displayGroup: DisplayGroup = DisplayGroup("dg", Leaf(EMPTY_LEAF))

    #: A repository of DisplayGroupInstance objects.
    displayGroupInstances: dict[str, DisplayGroupInstance] = {}

    #: An EngineeringFeatureDisplayOptions object.
    engineeringFeatureOptions: EngineeringFeatureDisplayOptions = EngineeringFeatureDisplayOptions()

    #: A PredefinedFieldDisplayOptions object.
    predefinedFieldOptions: PredefinedFieldDisplayOptions = PredefinedFieldDisplayOptions()

    #: A GeometricRestrictionDisplayOptions object.
    geometricRestrictionOptions: GeometricRestrictionDisplayOptions = GeometricRestrictionDisplayOptions()

    #: A GeometryDisplayOptions object.
    geometryOptions: GeometryDisplayOptions = GeometryDisplayOptions()

    #: An InteractionDisplayOptions object.
    interactionOptions: InteractionDisplayOptions = InteractionDisplayOptions()

    #: A LoadDisplayOptions object.
    loadOptions: LoadDisplayOptions = LoadDisplayOptions()

    #: A MeshDisplayOptions object.
    meshOptions: MeshDisplayOptions = MeshDisplayOptions()

    #: An OptimizationTaskDisplayOptions object.
    optimizationTaskOptions: OptimizationTaskDisplayOptions = OptimizationTaskDisplayOptions()

    #: A StopConditionDisplayOptions object.
    stopConditionOptions: StopConditionDisplayOptions = StopConditionDisplayOptions()

    #: A SymbolDisplayOptions object.
    symbolOptions: SymbolDisplayOptions = SymbolDisplayOptions()

    #: A tuple of Strings specifying the names of the part instances that are visible in the
    #: viewport. The default value is an empty sequence.
    visibleInstances: tuple = ()

    #: A String specifying the step for which objects are to be displayed. Possible values are
    #: any valid step name. The default value is "Initial".
    step: str = ""

    @abaqus_method_doc
    def setValues(
        self,
        visibleInstances: tuple = (),
        step: str = "",
        renderStyle: Literal[C.SHADED, C.FILLED, C.WIREFRAME, C.HIDDEN] = WIREFRAME,
        mesh: Boolean = OFF,
        loads: Boolean = OFF,
        bcs: Boolean = OFF,
        interactions: Boolean = OFF,
        constraints: Boolean = OFF,
        connectors: Boolean = OFF,
        cnxEndPoints: Boolean = ON,
        cnxLocalAxes: Boolean = ON,
        cnxTypeLabels: Boolean = ON,
        cnxTagDisplay: Boolean = OFF,
        predefinedFields: Boolean = OFF,
        visibleDisplayGroups: Sequence[DisplayGroup] = (),
        engineeringFeatures: Boolean = OFF,
        renderBeamProfiles: Boolean = OFF,
        beamScaleFactor: float = 1,
        optimizationTasks: Boolean = OFF,
        geometricRestrictions: Boolean = OFF,
        stopConditions: Boolean = OFF,
    ):
        """This method modifies the AssemblyDisplayOptions object.

        Parameters
        ----------
        visibleInstances
            A sequence of Strings specifying the names of the part instances that are visible in the
            viewport. The default value is an empty sequence.
        step
            A String specifying the step for which objects are to be displayed. Possible values are
            any valid step name. The default value is "Initial".
        renderStyle
            A SymbolicConstant specifying how the image in the viewport is rendered. Possible values
            are WIREFRAME, HIDDEN, SHADED, and FILLED. The default value is WIREFRAME.
        mesh
            A Boolean specifying whether the mesh is shown. The default value is OFF.
        loads
            A Boolean specifying whether loads are shown. The default value is OFF.
        bcs
            A Boolean specifying whether boundary conditions are shown. The default value is OFF.
        interactions
            A Boolean specifying whether interactions are shown. The default value is OFF.
        constraints
            A Boolean specifying whether constraints are shown. The default value is OFF.
        connectors
            A Boolean specifying whether connectors are shown. The default value is OFF.
        cnxEndPoints
            A Boolean specifying whether the connector end points are shown. This member is
            applicable only if **connectors** = ON. The default value is ON.
        cnxLocalAxes
            A Boolean specifying whether the connector local coordinate system axes are shown. This
            member is applicable only if **connectors** = ON. The default value is ON.
        cnxTypeLabels
            A Boolean specifying whether the connector section type labels are shown. This member is
            applicable only if **connectors** = ON. The default value is ON.
        cnxTagDisplay
            A Boolean specifying whether the tag information is displayed along with the connector
            section type labels. This member is applicable only if **connectors** = ON and if
            **cnxTypeLabels** = ON. The default value is OFF.
        predefinedFields
            A Boolean specifying whether fields and initial conditions are shown. The default value
            is OFF.
        visibleDisplayGroups
            A sequence of DisplayGroup objects specifying the DisplayGroups visible in the viewport.
            Currently the sequence can contain a maximum of one DisplayGroup object. The default
            value is an empty sequence.
        engineeringFeatures
            A Boolean specifying whether to display engineering features. The default value is OFF.
        renderBeamProfiles
            A Boolean specifying whether to render the beam profiles. The default value is OFF.
        beamScaleFactor
            A Float specifying the beam profile scale factor. The beamScaleFactor must be greater
            than zero. The default value is 1.0.
        optimizationTasks
            A Boolean specifying whether optimization tasks are shown. The default value is OFF.
        geometricRestrictions
            A Boolean specifying whether geometric restrictions are shown. The default value is OFF.
        stopConditions
            A Boolean specifying whether stop conditions are shown. The default value is OFF.

        Raises
        ------
        RangeError
        """
        ...



from __future__ import annotations

from abqpy.decorators import abaqus_class_doc

from ..Adaptivity.AdaptiveMeshConstraintState import AdaptiveMeshConstraintState
from ..Adaptivity.AdaptiveMeshDomain import AdaptiveMeshDomain
from ..BoundaryCondition.BoundaryConditionState import BoundaryConditionState
from ..Load.LoadCase import LoadCase
from ..Load.LoadState import LoadState
from ..PredefinedField.PredefinedFieldState import PredefinedFieldState
from ..StepMiscellaneous.Control import Control
from ..StepMiscellaneous.SolverControl import SolverControl
from ..StepOutput.DiagnosticPrint import DiagnosticPrint
from ..StepOutput.FieldOutputRequestState import FieldOutputRequestState
from ..StepOutput.HistoryOutputRequestState import HistoryOutputRequestState
from ..StepOutput.Monitor import Monitor
from ..StepOutput.Restart import Restart
from ..UtilityAndView.abaqusConstants import OFF, Boolean, SymbolicConstant
from .AnalysisStep import AnalysisStep


@abaqus_class_doc
class InitialStep(AnalysisStep):
    """The InitialStep object is a placeholder that you cannot create, delete, or modify. The InitialStep object
    exists in every model by default and is used to allow the preexisting boundary conditions and interactions
    to be defined in the model. The InitialStep object is derived from the AnalysisStep object.

    .. note::
        This object can be accessed by::

            import step
            mdb.models[name].steps[name]
    """

    #: A String specifying the repository key.
    name: str = ""

    #: A SymbolicConstant specifying whether the step has an explicit procedure type
    #: (*procedureType* = ANNEAL, DYNAMIC_EXPLICIT, or DYNAMIC_TEMP_DISPLACEMENT).
    explicit: SymbolicConstant

    #: A Boolean specifying whether the step has a perturbation procedure type.
    perturbation: Boolean = OFF

    #: A Boolean specifying whether the step has a mechanical procedure type.
    nonmechanical: Boolean = OFF

    #: A SymbolicConstant specifying the Abaqus procedure. Possible values are:
    #:
    #: - ANNEAL
    #: - BUCKLE
    #: - COMPLEX_FREQUENCY
    #: - COUPLED_TEMP_DISPLACEMENT
    #: - COUPLED_THERMAL_ELECTRIC
    #: - DIRECT_CYCLIC
    #: - DYNAMIC_IMPLICIT
    #: - DYNAMIC_EXPLICIT
    #: - DYNAMIC_SUBSPACE
    #: - DYNAMIC_TEMP_DISPLACEMENT
    #: - COUPLED_THERMAL_ELECTRICAL_STRUCTURAL
    #: - FREQUENCY
    #: - GEOSTATIC
    #: - HEAT_TRANSFER
    #: - MASS_DIFFUSION
    #: - MODAL_DYNAMICS
    #: - RANDOM_RESPONSE
    #: - RESPONSE_SPECTRUM
    #: - SOILS
    #: - STATIC_GENERAL
    #: - STATIC_LINEAR_PERTURBATION
    #: - STATIC_RIKS
    #: - STEADY_STATE_DIRECT
    #: - STEADY_STATE_MODAL
    #: - STEADY_STATE_SUBSPACE
    #: - VISCO
    procedureType: SymbolicConstant

    #: A Boolean specifying whether the step is suppressed or not. The default value is OFF.
    suppressed: Boolean = OFF

    #: A repository of FieldOutputRequestState objects.
    fieldOutputRequestState: dict[str, FieldOutputRequestState] = {}

    #: A repository of HistoryOutputRequestState objects.
    historyOutputRequestState: dict[str, HistoryOutputRequestState] = {}

    #: A DiagnosticPrint object.
    diagnosticPrint: DiagnosticPrint = DiagnosticPrint()

    #: A Monitor object.
    monitor: Monitor | None = None

    #: A Restart object.
    restart: Restart = Restart()

    #: A repository of AdaptiveMeshConstraintState objects.
    adaptiveMeshConstraintStates: dict[str, AdaptiveMeshConstraintState] = {}

    #: A repository of AdaptiveMeshDomain objects.
    adaptiveMeshDomains: dict[str, AdaptiveMeshDomain] = {}

    #: A Control object.
    control: Control = Control()

    #: A SolverControl object.
    solverControl: SolverControl = SolverControl()

    #: A repository of BoundaryConditionState objects.
    boundaryConditionStates: dict[str, BoundaryConditionState] = {}

    #: A repository of InteractionState objects.
    interactionStates: int | None = None

    #: A repository of LoadState objects.
    loadStates: dict[str, LoadState] = {}

    #: A repository of LoadCase objects.
    loadCases: dict[str, LoadCase] = {}

    #: A repository of PredefinedFieldState objects.
    predefinedFieldStates: dict[str, PredefinedFieldState] = {}



from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ...UtilityAndView.abaqusConstants import GENERAL, ISOTROPIC, OFF, Boolean
from ...UtilityAndView.abaqusConstants import abaqusConstants as C
from .PressureEffect import PressureEffect
from .SoretEffect import SoretEffect


@abaqus_class_doc
class Diffusivity:
    r"""The Diffusivity object specifies mass diffusivity.

    .. note::
        This object can be accessed by::

            import material
            mdb.models[name].materials[name].diffusivity
            import odbMaterial
            session.odbs[name].materials[name].diffusivity

        The table data for this object are:

        - If **type** = ISOTROPIC, the table data specify the following:

            - Diffusivity, :math:`D`.
            - Concentration, :math:`c`.
            - Temperature, if the data depend on temperature.
            - Value of the first field variable, if the data depend on field variables.
            - Value of the second field variable.
            - Etc.
        - If **type** = ORTHOTROPIC, the table data specify the following:

            - :math:`D_{11}`.
            - :math:`D_{22}`.
            - :math:`D_{33}`.
            - Concentration, :math:`c`.
            - Temperature, if the data depend on temperature.
            - Value of the first field variable, if the data depend on field variables.
            - Value of the second field variable.
            - Etc.
        - If **type** = ANISOTROPIC, the table data specify the following:

            - :math:`D_{11}`.
            - :math:`D_{12}`.
            - :math:`D_{22}`.
            - :math:`D_{13}`.
            - :math:`D_{23}`.
            - :math:`D_{33}`.
            - Concentration, :math:`c`.
            - Temperature, if the data depend on temperature.
            - Value of the first field variable, if the data depend on field variables.
            - Value of the second field variable.
            - Etc.

        The corresponding analysis keywords are:

        - DIFFUSIVITY
    """

    #: A PressureEffect object.
    pressureEffect: PressureEffect = PressureEffect(((),))

    #: A SoretEffect object.
    soretEffect: SoretEffect = SoretEffect(((),))

    @abaqus_method_doc
    def __init__(
        self,
        table: tuple,
        type: Literal[C.ANISOTROPIC, C.ISOTROPIC, C.ORTHOTROPIC] = ISOTROPIC,
        law: Literal[C.GENERAL, C.FICK] = GENERAL,
        temperatureDependency: Boolean = OFF,
        dependencies: int = 0,
    ):
        """This method creates a Diffusivity object.

        .. note::
            This function can be accessed by::

                mdb.models[name].materials[name].Diffusivity
                session.odbs[name].materials[name].Diffusivity

        Parameters
        ----------
        table
            A sequence of sequences of Floats specifying the items described below.
        type
            A SymbolicConstant specifying the type of diffusivity. Possible values are ISOTROPIC,
            ORTHOTROPIC, and ANISOTROPIC. The default value is ISOTROPIC.
        law
            A SymbolicConstant specifying the diffusion behavior. Possible values are GENERAL and
            FICK. The default value is GENERAL.
        temperatureDependency
            A Boolean specifying whether the data depend on temperature. The default value is OFF.
        dependencies
            An Int specifying the number of field variable dependencies. The default value is 0.

        Returns
        -------
        Diffusivity
            A Diffusivity object.

        Raises
        ------
        RangeError
        """
        ...

    @abaqus_method_doc
    def setValues(self, *args, **kwargs):
        """This method modifies the Diffusivity object.

        Raises
        ------
        RangeError
        """
        ...



from typing import Dict

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc


@abaqus_class_doc
class Repository(Dict):
    """Repositories are containers that store a particular type of object; for example, the steps repository
    contains all the steps defined in the model. An Abaqus Scripting Interface Repository maps a key to a value.
    The key is usually a String, and the value is any Python object, usually an Abaqus object. A repository is
    similar to a Python dictionary; however, only a constructor can add an object to a repository. In addition,
    all of the objects in a repository are of the same base type. For more information, see Repositories. A
    Repository has no constructor. Abaqus creates empty repositories when you import a module. For example,
    Abaqus creates an empty parts repository when you import the part module. The following methods of the
    Repository object are standard Python dictionary methods and are not described here:

    The Repository object is derived from the dict object.
    - has_key
    - items
    - keys
    - values
    -
    [changeKey(...)](https://help.3ds.com/2022/english/DSSIMULIA_Established/SIMACAEKERRefMap/simaker-c-utlrepositorypyc.htm?ContextScope=all#simaker-utlrepositorychangekeypyc)
    """

    @abaqus_method_doc
    def changeKey(self, fromName: str, toName: str):
        """This method changes the name of a key in a repository and the **name** member of the value object.

        Parameters
        ----------
        fromName
            A String specifying the old name of the repository key.
        toName
            A String specifying the new name of the repository key.
        """
        ...



from __future__ import annotations

from abqpy.decorators import abaqus_class_doc

from ..UtilityAndView.abaqusConstants import SymbolicConstant
from .InteractionState import InteractionState


@abaqus_class_doc
class ElasticFoundationState(InteractionState):
    """The ElasticFoundationState object stores the propagating data for an ElasticFoundation object. One
    instance of this object is created internally by the ElasticFoundation object for each step. The instance is
    also deleted internally by the ElasticFoundation object. The ElasticFoundationState object has no
    constructor or methods. The ElasticFoundationState object is derived from the InteractionState object.

    .. note::
        This object can be accessed by::

            import interaction
            mdb.models[name].steps[name].interactionStates[name]
    """

    #: A Float specifying the foundation stiffness per area.
    stiffness: float | None = None

    #: A SymbolicConstant specifying the propagation state of the stiffness member. Possible
    #: values are UNSET, SET, UNCHANGED, and FREED.
    stiffnessState: SymbolicConstant

    #: A SymbolicConstant specifying the propagation state of the InteractionState object.
    #: Possible values are:
    #:
    #: - NOT_YET_ACTIVE
    #: - CREATED
    #: - PROPAGATED
    #: - MODIFIED
    #: - DEACTIVATED
    #: - NO_LONGER_ACTIVE
    #: - TYPE_NOT_APPLICABLE
    #: - INSTANCE_NOT_APPLICABLE
    #: - BUILT_INTO_BASE_STATE
    status: SymbolicConstant



from __future__ import annotations

from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..Region.Region import Region
from ..UtilityAndView.abaqusConstants import UNIFORM, UNSET, SymbolicConstant
from ..UtilityAndView.abaqusConstants import abaqusConstants as C
from .Load import Load


@abaqus_class_doc
class BodyCharge(Load):
    """The BodyCharge object stores the data for a body charge. The BodyCharge object is derived from the Load
    object.

    .. note::
        This object can be accessed by::

            import load
            mdb.models[name].loads[name]
    """

    #: A String specifying the load repository key.
    name: str = ""

    #: A SymbolicConstant specifying how the load is distributed spatially. Possible values are
    #: UNIFORM and FIELD. The default value is UNIFORM.
    distributionType: SymbolicConstant = UNIFORM

    #: A String specifying the name of the AnalyticalField object associated with this load.
    #: The **field** argument applies only when **distributionType** = FIELD. The default value is an
    #: empty string.
    field: str = ""

    #: A Region object specifying the region to which the load is applied.
    region: Region = Region()

    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        createStepName: str,
        region: Region,
        magnitude: float,
        amplitude: str = UNSET,
        distributionType: Literal[C.FIELD, C.UNIFORM] = UNIFORM,
        field: str = "",
    ):
        """This method creates a BodyCharge object.

        .. note::
            This function can be accessed by::

                mdb.models[name].BodyCharge

        Parameters
        ----------
        name
            A String specifying the load repository key.
        createStepName
            A String specifying the name of the step in which the load is created. This must be the
            first analysis step name.
        region
            A Region object specifying the region to which the load is applied.
        magnitude
            A Float specifying the load magnitude.
        amplitude
            A String or the SymbolicConstant UNSET specifying the name of the amplitude reference.
            UNSET should be used if the load has no amplitude reference. The default value is UNSET.
            You should provide the **amplitude** argument only if it is valid for the specified step.
        distributionType
            A SymbolicConstant specifying how the load is distributed spatially. Possible values are
            UNIFORM and FIELD. The default value is UNIFORM.
        field
            A String specifying the name of the AnalyticalField object associated with this load.
            The **field** argument applies only when **distributionType** = FIELD. The default value is an
            empty string.

        Returns
        -------
        BodyCharge
            A BodyCharge object.
        """
        super().__init__()

    @abaqus_method_doc
    def setValues(
        self,
        amplitude: str = UNSET,
        distributionType: Literal[C.FIELD, C.UNIFORM] = UNIFORM,
        field: str = "",
    ):
        """This method modifies the data for an existing BodyCharge object in the step where it is created.

        Parameters
        ----------
        amplitude
            A String or the SymbolicConstant UNSET specifying the name of the amplitude reference.
            UNSET should be used if the load has no amplitude reference. The default value is UNSET.
            You should provide the **amplitude** argument only if it is valid for the specified step.
        distributionType
            A SymbolicConstant specifying how the load is distributed spatially. Possible values are
            UNIFORM and FIELD. The default value is UNIFORM.
        field
            A String specifying the name of the AnalyticalField object associated with this load.
            The **field** argument applies only when **distributionType** = FIELD. The default value is an
            empty string.
        """
        ...

    @abaqus_method_doc
    def setValuesInStep(self, stepName: str, magnitude: float | None = None, amplitude: str = ""):
        """This method modifies the propagating data for an existing BodyCharge object in the specified step.

        Parameters
        ----------
        stepName
            A String specifying the name of the step in which the load is modified.
        magnitude
            A Float specifying the load magnitude.
        amplitude
            A String or a SymbolicConstant specifying the name of the amplitude reference. Possible
            values for the SymbolicConstant are UNCHANGED and FREED. UNCHANGED should be used if the
            amplitude is propagated from the previous static analysis step. FREED should be used if
            the load is changed to have no amplitude reference. You should provide the **amplitude**
            argument only if it is valid for the specified step.
        """
        ...
